/*
SYBTOOL - a cool tool to access SYBASE server

Copyright (C) 1994, Ivan Kiselev

Everything   not  already     copyrighted  by  Brad      Appleton (see
./CmdLine/README) or  Chris    Thewalt (see   ./LineEdit/getline.c) is
copyrighted by Ivan Kiselev.

SYBTOOL software,   both  binary and source   (hereafter, Software) is
copyrighted by Ivan Kiselev, and ownership remains with Ivan Kiselev.

Ivan  Kiselev grants you  (hereafter, Licensee)  a  license to use the
Software for academic,  research and internal business  purposes only,
without a fee.  Licensee may distribute the binary and source code (if
released) to third parties provided that the copyright notice and this
statement appears on all copies and that no  charge is associated with
such copies.

Licensee may make derivative works.   However, if Licensee distributes
any  derivative  work  based on  or  derived  from the  Software, then
Licensee will (1) notify Ivan   Kiselev regarding its distribution  of
the derivative work, and (2) clearly notify users that such derivative
work is a modified version and not the original SYBTOOL distributed by
Ivan Kiselev.

Any Licensee  wishing to make  commercial use  of the Software  should
contact  Ivan Kiselev  to negotiate  an  appropriate license for  such
commercial  use.  Commercial  use includes  (1) integration  of all or
part of the  source code into a product  for sale or  license by or on
behalf of Licensee to third parties, or (2) distribution of the binary
code or source   code  to third parties   that  need it   to utilize a
commercial product sold or licensed by or on behalf of Licensee.

IVAN KISELEV MAKES NO REPRESENTATIONS   ABOUT THE SUITABILITY OF  THIS
SOFTWARE FOR ANY  PURPOSE.  IT IS PROVIDED  "AS IS" WITHOUT EXPRESS OR
IMPLIED WARRANTY.  IVAN KISELEV  SHALL NOT BE  LIABLE FOR  ANY DAMAGES
SUFFERED BY THE USERS OF THIS SOFTWARE.

By  using or  copying this Software,  Licensee agrees  to abide by the
copyright law and all other applicable laws of the U.S. including, but
not  limited to, export  control laws, and  the terms of this license.
Ivan     Kiselev shall  have   the  right   to terminate  this license
immediately   by   written notice   upon   Licensee's  breach  of,  or
non-compliance with, any of its  terms.  Licensee may be held  legally
responsible    for  any  copyright   infringement  that is   caused or
encouraged    by Licensee's failure   to abide  by  the  terms of this
license.

Comments and questions are welcome and can be sent to ivank@crt.com or
ivank@mcs.com

Postal address:
Ivan Kiselev
441 Elm
Deerfield, IL 60015

*/

#ifndef _A_STRING_
#define _A_STRING_

#include <string.h>
#include <new.h>

class String
{
public:

    // Constructors
    String()
    {
	*(ptr = new char[1]) = '\0';
    };

    String( const String &aString)
    {
	::strcpy(ptr = new char[::strlen(aString.ptr)+1], aString.ptr);
    };

    String( const char *s)
    {
	::strcpy(ptr = new char[::strlen(s)+1], s);
    };

    // Destructor
    ~String()
    {
	delete [] ptr;
    };


    // Assignment operator
    String& operator=(const String &aString)
    {
	debug("String operator=(const String &)"); 
	if ( this != &aString )
	{
	    this->String::~String();
	    new (this) String(aString);
	}
	return *this;
    };
    
    // Comparisons
    int operator< (const String &aString) { return ::strcmp(ptr, aString.ptr) <  0; } ;
    int operator==(const String &aString) { return ::strcmp(ptr, aString.ptr) == 0; } ;

    int operator<=(const String &aString)
    {
	return (operator<(aString) || operator==(aString));
    } ;
    int operator>=(const String &aString) { return (!operator< (aString)); } ;
    int operator!=(const String &aString) { return (!operator==(aString)); } ;
    int operator> (const String &aString) { return (!operator<=(aString)); } ;

    // Conversion
    operator const char* () const { return ptr; } ;

    // Other operators
    String operator+(const String &aString)
    {
	char *buf = new char[::strlen(ptr)+::strlen(aString.ptr)+1];
	::strcpy(buf, ptr);
	::strcat(buf, aString.ptr);

	return String(buf);
    };

    String &operator+=(const String &aString)
    {
	char *buf = new char[::strlen(ptr)+::strlen(aString.ptr)+1];
	::strcpy(buf, ptr);
	::strcat(buf, aString.ptr);

	delete [] ptr;
	ptr = buf;
	return *this;
    };

    // I/O operators
    friend ostream & operator<<(ostream&, const String&);
    friend istream & operator>>(istream&, String&);

    // Misc.
    int length() { return ::strlen(ptr); };
    String &chop() { ptr[::strlen(ptr)-1] = '\0'; return *this; };
    int blank();

private:
    char *ptr;
};

ostream & operator<<(ostream &aStream, const String &aString)
{
    return ( aStream << (const char *)aString );
};

istream & operator>>(istream &aStream, String &aString)
{
    char buf[4096];
    buf[0] = '\0';

    aStream.getline(buf, 4096);

    aString = buf;

    return aStream;
};

int String::blank()
{
    char *s = ptr;
    while(isspace(*s++));
    return ( *s == '\0' );
}

#endif // _A_STRING_


