/*
SYBTOOL - a cool tool to access SYBASE server

Copyright (C) 1994, Ivan Kiselev

Everything   not  already     copyrighted  by  Brad      Appleton (see
./CmdLine/README) or  Chris    Thewalt (see   ./LineEdit/getline.c) is
copyrighted by Ivan Kiselev.

SYBTOOL software,   both  binary and source   (hereafter, Software) is
copyrighted by Ivan Kiselev, and ownership remains with Ivan Kiselev.

Ivan  Kiselev grants you  (hereafter, Licensee)  a  license to use the
Software for academic,  research and internal business  purposes only,
without a fee.  Licensee may distribute the binary and source code (if
released) to third parties provided that the copyright notice and this
statement appears on all copies and that no  charge is associated with
such copies.

Licensee may make derivative works.   However, if Licensee distributes
any  derivative  work  based on  or  derived  from the  Software, then
Licensee will (1) notify Ivan   Kiselev regarding its distribution  of
the derivative work, and (2) clearly notify users that such derivative
work is a modified version and not the original SYBTOOL distributed by
Ivan Kiselev.

Any Licensee  wishing to make  commercial use  of the Software  should
contact  Ivan Kiselev  to negotiate  an  appropriate license for  such
commercial  use.  Commercial  use includes  (1) integration  of all or
part of the  source code into a product  for sale or  license by or on
behalf of Licensee to third parties, or (2) distribution of the binary
code or source   code  to third parties   that  need it   to utilize a
commercial product sold or licensed by or on behalf of Licensee.

IVAN KISELEV MAKES NO REPRESENTATIONS   ABOUT THE SUITABILITY OF  THIS
SOFTWARE FOR ANY  PURPOSE.  IT IS PROVIDED  "AS IS" WITHOUT EXPRESS OR
IMPLIED WARRANTY.  IVAN KISELEV  SHALL NOT BE  LIABLE FOR  ANY DAMAGES
SUFFERED BY THE USERS OF THIS SOFTWARE.

By  using or  copying this Software,  Licensee agrees  to abide by the
copyright law and all other applicable laws of the U.S. including, but
not  limited to, export  control laws, and  the terms of this license.
Ivan     Kiselev shall  have   the  right   to terminate  this license
immediately   by   written notice   upon   Licensee's  breach  of,  or
non-compliance with, any of its  terms.  Licensee may be held  legally
responsible    for  any  copyright   infringement  that is   caused or
encouraged    by Licensee's failure   to abide  by  the  terms of this
license.

Comments and questions are welcome and can be sent to ivank@crt.com or
ivank@mcs.com

Postal address:
Ivan Kiselev
441 Elm
Deerfield, IL 60015

*/

#include <iostream.h>
#include <fstream.h>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <termios.h>

#include <debug.H>

#include <Stack.H>
#include <String.H>


// CmdLine library
#include <cmdargs.h>

// SYBASE stuff

#define COMPILE_STYLE	CPP_COMPILE	// Compile as C++ source
#define VOID		void		// Missed from *.h (Sybase's error)

typedef int (*SybaseHandler)();		// This it a back door way

extern  "C"
{
#include <sybfront.h>
#include <sybdb.h>
}

// Forward declarations of error handlers
int err_handler(DBPROCESS *, int, int, int, char *, char *);
int msg_handler(DBPROCESS *, DBINT, int, int, char *, char *, char *, DBUSMALLINT);


// Line Edit library
extern "C" char *getline(char *);
extern "C" void gl_histadd(const char *);
extern "C" void gl_setwidth(int );
extern "C" int (*gl_in_hook)(char *);

extern "C" int isatty(int);

#ifndef HIST_SIZE
#define HIST_SIZE 100
#endif

#ifndef HIST_FILE
#define HIST_FILE "/.sybtool"
#endif

#ifndef VERSION
#define VERSION 1.3
#endif



#define COMMAND_BUFFER_SIZE		2048


// Recognize commands 
int IsCommand(const char *, const char *);

int setTerminalWidth(char *)
{
    struct winsize ws;		// This shit is for ioctl()

    ioctl (0, TIOCGWINSZ, &ws);
    gl_setwidth(ws.ws_col);

    return -1;			// It means no changes were made to the buffer
}

main(int argc, char * argv[])
{
    // Declare some variables
    char *progName = "sybtool";		// Name of the program
    int	IsTerminal = isatty(0);		// Determine a type of input stream

    Stack<String> histBuffer(HIST_SIZE);// Copy of the editing buffer (list of strings)

    // Print the copyright notice
    if(IsTerminal)
    {
	cout << endl;
	cout << "SYBTOOL ver." << VERSION << " - cool tool to access Sybase server." << endl;
	cout << "Copyright (C) 1994 by Ivan Kiselev" << endl;
	cout << endl;
    }

    
    //
    // Deal with command-line arguments
    //

    // Declare arguments
    CmdArgStr  user('U', "user", "user", "User ID", CmdArg::isREQ);
    CmdArgStr  password('P', "password", "password", "User's password", CmdArg::isREQ);
    CmdArgStr  server('S', "server", "server", "Server to connect to ($DSQUERY or \"SYBASE\" is the default)");
    CmdArgStr  file('f', "file", "histtory_file", "History file name ($HOME/.sybtool is the default)");
    CmdArgStr  cmdend('c', "cmdend", "cmd_end", "Command terminator (\"go\" is the default)");
    CmdArgStr  ifile('I', "ifile", "interface_file", "Interface file ($SYBASE/interfaces is the default)");
    CmdArgStr  db('D', "database", "database", "Database to use.");
    CmdArgChar  colSeparator('s', "colseparator", "col_separator", "Column separator character (space bar is the default)");
    CmdArgBool  noHeader('H', "hoheader", "Turns header printing off");
    CmdArgBool  noPadding('p', "hopadding", "Turns padding off");
    

    // Declare the command and the argument-iterator
    CmdLine  cmd(*argv, &user, &password, &server, &file, &cmdend, &ifile, &db, &colSeparator, &noHeader, &noPadding, NULL);
    CmdArgvIter  arg_iter(--argc, ++argv);

    // Set default separator
    colSeparator = ' ';

    // Parse arguments
    cmd.parse(arg_iter);

    // Deal with history file name

    String fileName;

    if (file.isNULL())
    {
	// If not specified in command line - set to default one
	fileName = String(getenv("HOME")) + String(HIST_FILE);
    }
    else
    {
	// If specified...
	fileName = (const char *)file;
    }


    // Command terminator
    String commandTerminator = "go";
    if (!cmdend.isNULL())
    {
	commandTerminator = (const char *)cmdend;
    }
    

    //
    // Set all Sybase stuff
    //

    DBPROCESS      *dbproc;	/* Our connection with SQL Server. */
    LOGINREC       *login;	/* Our login information. */

    // Initialize DB-Library.
    if (dbinit() == FAIL)
    {
	cerr << progName << ": Cannot initialize SYBASE library." << endl;
	exit(1);
    }


    // Interface file business
    if (!ifile.isNULL())
    {
	dbsetifile((char *)(const char *)ifile);
    }
    

    // Set error handlers
    (void)dberrhandle((SybaseHandler)err_handler);
    (void)dbmsghandle((SybaseHandler)msg_handler);

    // Login
    if ( (login = dblogin()) == (LOGINREC *)NULL)
    {
	cerr << progName << ": Cannot allocate login record." << endl;
	exit(1);
    }

    DBSETLUSER(login, (char *)(const char *)user);
    DBSETLPWD(login, (char *)(const char *)password);
    DBSETLAPP(login, progName);


    // Connect to SYBASE server
    if ( (dbproc = dbopen(login, (char *)(const char *)server)) == (DBPROCESS *)NULL )
    {
	cerr << progName <<  ": Cannot connect to the SYBASE server." << endl;
	exit(1);
    }

    // Set the database to use if any
    if (!db.isNULL())
    {
	if ( dbuse(dbproc, (char *)(const char *)db) == FAIL)
	{
	    cerr << progName << ": Cannot set database." << endl;
	    exit(1);
	}
    }

    // Set some formatting parameters
    if (colSeparator != ' ')
    {
	char tmp[2]; tmp[0] = (char)colSeparator; tmp[1] = 0;
	
	if ( dbsetopt(dbproc, DBPRCOLSEP, tmp, 1) == FAIL)
	{
	    cerr << progName <<  ": Cannot set column separator." << endl;
	    exit(1);
	}
    }

    if ( noPadding )
    {
	if ( dbsetopt(dbproc, DBPRPAD, " ", DBPADOFF) == FAIL)
	{
	    cerr << progName <<  ": Cannot turn off padding." << endl;
	    exit(1);
	}
    }
	
    if ( dbsetopt(dbproc, DBPRLINELEN, "1048576", strlen("1048576")) == FAIL)
    {
	cerr << progName <<  ": Cannot set the line width." << endl;
	exit(1);
    }

    // Load the history
    if(IsTerminal)
    {
	fstream histFile(fileName, ios::in);	// History file (stream)

	// Reset terminal width each time
	gl_in_hook = setTerminalWidth;

	if (histFile)
	{
	    // Read the file
	    String tmp;
	    while(histFile >> tmp) histBuffer.push(tmp);
	    histFile.close();

	    // And put it into editing buffer
	    for( int i=0; i < histBuffer.count(); i++)
	    {
		gl_histadd(histBuffer[i]);
	    }
	}
	else
	{
	    cerr << "Cannot open file " << fileName << " for input." << endl;
	}
    }

    //
    // Here is the main cycle
    //

    char prompt[10];				// Running prompt
    int commandCount = 0;			// Current command number

    RETCODE ret_code;				// Return code holder

    // Begin looping
    while(1)
    {
	String tempString;

	// Get input line
	if(IsTerminal)
	{
	    // Format prompt
	    sprintf(prompt,"%05d>", commandCount++);

	    // Edit command 
	    tempString = getline(prompt);

	    // Exit if end of input stream
	    if (!tempString.length()) break;
	    
	    // Skip blank lines
	    if (tempString.blank()) continue;

	    // Get rid of NL
	    tempString.chop();

	    // Check for "exit" of "quit"
	    if( IsCommand(tempString, "exit") || IsCommand(tempString, "quit") ) break;

	    //
	    // Save the history
	    //
	    if(
	       ( !IsCommand(tempString, commandTerminator) )    // It's not terminator
	       &&
	       ( !IsCommand(tempString, "reset") )              // It's not "reset"
	       &&
	       (histBuffer[histBuffer.count()-1] != tempString) // It's not duplicate
	       )
		
	    {
		// Add it to the history buffer for future saving
		histBuffer.push(tempString);

		// Add it to the editing buffer
		gl_histadd(tempString);
	    }
	}
	else
	{
	    // Get it from input stream
	    if(!(cin >> tempString))
	    {
		break;
	    }

	    // Skip blank lines
	    if (tempString.blank()) continue;
	}

	// Look for "reset"
	if ( IsCommand(tempString, "reset") )
	{
	    dbfreebuf(dbproc);
	    continue;
	}

	// Look for end of batch
	if ( !IsCommand(tempString, commandTerminator) )
	{
	    // Set query
	    if (dbfcmd(dbproc, "%s\n", (const char *)tempString) == FAIL)
	    {
		cerr << progName << ": Cannot put command into command buffer." << endl;
		dbfreebuf(dbproc);
	    }
	    // Proceed to the next input line
	    continue;
	}
	else
	{
	    // Check for non-empty command buffer
	    if (!dbstrlen(dbproc))
	    {
		cerr << progName << ": Cannot execute empty command!" << endl;
		continue;
	    }

	    // Execute it!
	    if (dbsqlexec(dbproc) == FAIL)
	    {
		cerr << progName << ": Cannot execute. Command canceled!" << endl;

		// If in batch mode - exit
		if(!IsTerminal) exit(1);

		// Cancel if it's bad and go to the next batch
		dbcancel(dbproc);
		continue;
	    }
	}

	// Process all results
	while ( (ret_code = dbresults(dbproc)) != NO_MORE_RESULTS )
	{
	    debug("Results");
	    // Check for correct results
	    if ( ret_code == FAIL )
	    {
		cerr << progName << ": Cannot process results." << endl;

		// If in batch mode - exit
		if(!IsTerminal) exit(1);

		// Cancel if it's bad and go to the next batch
		dbcancel(dbproc);
		continue;
	    }

	    // Are there any actual rows
	    if ( DBROWS(dbproc) == FAIL )
	    {
		dbcancel(dbproc);
		continue;
	    }

	    // Print header if not turned off
	    if ( !noHeader )
	    {
		dbprhead(dbproc);
	    }

	    // Process rows
	    if (dbprrow(dbproc) == FAIL )
	    {
		cerr << progName << ": Error processing results." << endl;

		// If in batch mode - exit
		if(!IsTerminal) exit(1);

		// Cancel if it's bad and go to the next batch
		dbcancel(dbproc);
		continue;
	    }
	}
    }

    // Exit peacefully
    dbclose(dbproc);

    // Save the history
    if(IsTerminal)
    {
	fstream histFile(fileName, ios::out);	// History file (stream)

	if(histFile)
	{
	    for(int i = 0; i< histBuffer.count(); i++) histFile << histBuffer[i] << endl;
	    histFile.close();
	}
	else
	{
	    cerr << "Cannot open file " << fileName << " for output." << endl;
	}
    }

    exit(0);
}


// Does string contain only one word that is command
int IsCommand(const char *string, const char *command)
{
	// All pissible delimeters
	const char *delim = " \t\n";

	// Local static buffer - to be protected from "strtok"
	static char buffer[COMMAND_BUFFER_SIZE];
	strcpy(buffer, string);

	// Check if the first token is "command" and there are no more tokens
	if ( (strcasecmp(strtok(buffer, delim), command) == 0) && (strtok(NULL, delim) == NULL) )
	{
		return 1;
	}

	return 0;
}


int 
err_handler(DBPROCESS * dbproc, int severity, int , int oserr, char *dberrstr, char *oserrstr)
{
	// Do not pay attention to minor errors
	if (severity < 8)
		return (INT_CANCEL);

	// Too bad...
	if ((dbproc == NULL) || (DBDEAD(dbproc)))
		return (INT_EXIT);

	// And this is the default case
	cerr << "DB-Library error:\t" << dberrstr << endl;

	if (oserr != DBNOERR)
		cerr << "Operating-system error:\t" << oserrstr << endl;

	return (INT_CANCEL);
}

int
msg_handler(DBPROCESS *, DBINT msgno, int msgstate, int severity, char *msgtext, char *srvname, char *procname, DBUSMALLINT line)
{

    // Ignore annoing messages...
    if  (
	( msgno == 5701 )  ||	// change database
	( msgno == 5703 )  ||	// change language
	( msgno == 5704 )  	// change charset
	)
    {
	return(0);
    }


    // Get some details about serious ones...
    if (severity > 16)
    {

	cerr << "Message: " << msgno << "  Severity: " << severity << " State: " << msgstate << endl;

	if (strlen(srvname) > 0)
	    cerr << "Server: " <<  srvname << endl;

	if (strlen(procname) > 0)
	    cerr << "Procedure: " <<  procname << endl;

	if (line > 0)
	    cerr << "Line: " << line << endl;
    }


    // Otherwise - just print it
    cerr << msgtext << endl;

    return (0);
}
